@@ -36,6 +36,7 @@ DATASET_INFO_REQUIRED_KEYS = (
     "creator_username",
     "frozen_at",
     "annotations",
+    "tags",
 )
 
 
@@ -58,8 +59,8 @@ def _get_base_uri_obj(base_uri):
 
 
 def _dict_to_mongo_query(query_dict):
-    valid_keys = ["free_text", "creator_usernames", "base_uris"]
-    list_keys = ["creator_usernames", "base_uris"]
+    valid_keys = ["free_text", "creator_usernames", "base_uris", "tags"]
+    list_keys = ["creator_usernames", "base_uris", "tags"]
 
     def _sanitise(query_dict):
         for key in list(query_dict.keys()):
@@ -76,6 +77,12 @@ def _dict_to_mongo_query(query_dict):
         else:
             return {"$or": [{key: v} for v in l]}
 
+    def _deal_with_possible_and_statement(l, key):
+        if len(l) == 1:
+            return {key: l[0]}
+        else:
+            return {key: {"$all": l}}
+
     _sanitise(query_dict)
 
     sub_queries = []
@@ -95,6 +102,13 @@ def _dict_to_mongo_query(query_dict):
                 "base_uri"
             )
         )
+    if "tags" in query_dict:
+        sub_queries.append(
+            _deal_with_possible_and_statement(
+                query_dict["tags"],
+                "tags"
+            )
+        )
 
     if len(sub_queries) == 0:
         return {}
@@ -143,6 +157,10 @@ def generate_dataset_info(dataset, base_uri):
         annotations[annotation_name] = dataset.get_annotation(annotation_name)
     dataset_info["annotations"] = annotations
 
+    # Add the tags.
+    tags = dataset.list_tags()
+    dataset_info["tags"] = tags
+
     # Clean up datetime.data.
     dataset_info_json_str = json.dumps(dataset_info, default=_json_serial)
     dataset_info = json.loads(dataset_info_json_str)
@@ -297,19 +315,21 @@ def summary_of_datasets_by_user(username):
     Return dictionary of summary information.
     Raises AuthenticationError if user is invalid.
     """
-    def _unique(l):
-        return list(set(l))
 
     # Get all the datasets the user has access to.
     datasets = search_datasets_by_user(username, query={})
+
     datasets_per_creator = {}
     datasets_per_base_uri = {}
+    datasets_per_tag = {}
 
     for ds in datasets:
         user = ds["creator_username"]
         uri = ds["base_uri"]
         datasets_per_creator[user] = datasets_per_creator.get(user, 0) + 1
         datasets_per_base_uri[uri] = datasets_per_base_uri.get(uri, 0) + 1
+        for tag in ds["tags"]:
+            datasets_per_tag[tag] = datasets_per_tag.get(tag, 0) + 1
 
     summary = {
         "number_of_datasets": len(datasets),
@@ -317,6 +337,8 @@ def summary_of_datasets_by_user(username):
         "base_uris": sorted(datasets_per_base_uri.keys()),
         "datasets_per_creator": datasets_per_creator,
         "datasets_per_base_uri": datasets_per_base_uri,
+        "tags": sorted(datasets_per_tag.keys()),
+        "datasets_per_tag": datasets_per_tag,
     }
 
     return summary
@@ -562,13 +584,6 @@ def get_admin_metadata_from_uri(uri):
     return dataset.as_dict()
 
 
-def get_readme_from_uri(uri):
-    """Return the readme information."""
-    collection = mongo.db[MONGO_COLLECTION]
-    item = collection.find_one({"uri": uri})
-    return item["readme"]
-
-
 def get_readme_from_uri_by_user(username, uri):
     """Return the readme.
 
